/*
   orxonox - the future of 3D-vertical-scrollers

   Copyright (C) 2004 orx

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   ### File Specific:
   main-programmer: Benjamin Grauer
   co-programmer: Patrick Boenzli
*/

#define DEBUG_SPECIAL_MODULE DEBUG_MODULE_LOAD

#include "new_resource_manager.h"
#include "debug.h"

#include <algorithm>
#include <assert.h>

//ObjectListDefinition(NewResourceManager);


//! Singleton Reference to the NewResourceManager
NewResourceManager* NewResourceManager::_singletonRef = NULL;


/**
 * @brief standard constructor
*/
NewResourceManager::NewResourceManager ()
{
//  this->registerObject(this, NewResourceManager::_objectList);
  this->setName("NewResourceManager");

  //this->dataDir = "./";
}


/**
 * @brief standard destructor
*/
NewResourceManager::~NewResourceManager ()
{
  // deleting the Resources-List
  //this->unloadAllByPriority(RP_GAME);

//   if (!this->resourceList.empty())
//     PRINTF(1)("Not removed all Resources, since there are still %d resources registered\n", this->resourceList.size());

  NewResourceManager::_singletonRef = NULL;
}




void NewResourceManager::registerType(Resources::Type* type)
{
  if(type->id() == -1)
  {
    type->setID(this->_resourceTypes.size());
    this->_resourceTypes.push_back(type);
    PRINTF(5)("ResourceType '%s' with ID %d added\n", type->storedClassName().c_str(), type->id());
  }
}

void NewResourceManager::setMainGlobalPath(const Directory& directory)
{
  this->_mainGlobalPath = directory;
  this->_mainGlobalPath.open();
}

void NewResourceManager::addGlobalPath(const Directory& directory)
{
  std::vector<Directory>::const_iterator it = std::find(this->_globalPaths.begin(), this->_globalPaths.end(), directory);
  if (it == this->_globalPaths.end())
    this->_globalPaths.push_back(directory);
}


bool NewResourceManager::addResourcePath(const std::string& resourceName, const std::string& pathName)
{
  std::vector<Resources::Type*>::iterator it;
  for (it = this->_resourceTypes.begin(); it != this->_resourceTypes.end(); ++it)
    if (*(*it) == resourceName)
      return (*it)->addResourcePath(pathName);
  PRINTF(2)("ResourcePath %s could not be added to the ResourceType %s\n", pathName.c_str(), resourceName.c_str());
  return false;
}

bool NewResourceManager::addResourceSubPath(const std::string& resourceName, const std::string& pathName)
{
  std::vector<Resources::Type*>::iterator it;
  for (it = this->_resourceTypes.begin(); it != this->_resourceTypes.end(); ++it)
    if (*(*it) == resourceName)
      return (*it)->addResourceSubPath(pathName);
  PRINTF(2)("ResourceSubPath %s could not be added to the ResourceType %s\n", pathName.c_str(), resourceName.c_str());
  return false;
}


/**
 * @brief outputs debug information about the NewResourceManager
 */
void NewResourceManager::debug() const
{
  PRINT(0)("=RM===================================\n");
  PRINT(0)("= RESOURCE-MANAGER DEBUG INFORMATION =\n");
  PRINT(0)("======================================\n");
  PRINT(0)("Listing %d Types: \n", this->_resourceTypes.size());
  std::vector<Resources::Type*>::const_iterator it;
  for (it = this->_resourceTypes.begin(); it != this->_resourceTypes.end(); ++it)
    (*it)->debug();

  PRINT(0)("==================================RM==\n");
}
